#include "stdafx.h"
#include "WallpaperQ.h"
#include "WallpaperQDlg.h"
#include "registry.h"
#include <time.h>
#include <stdlib.h>
#include "OptionsDialog.h"
#include "utility.h"
#include "AutoUpdater.h"
#include "MainHelperQ.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

Helper m_Helper;

extern "C"
{
	typedef BOOL (*LPSetHook)(HWND hWnd);
	typedef BOOL (*LPClearHook)(HWND hWnd);
}

/***************************************/
/* CAboutDlg dialog used for App About */
/***************************************/
class CAboutDlg : public CDialog
{
public:
	CAboutDlg();
protected:
	virtual BOOL OnInitDialog();
};

CAboutDlg::CAboutDlg() : CDialog(IDD_ABOUTBOX)
{
}

BOOL CAboutDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	return TRUE;
}

/*************************/
/* CWallpaperQDlg dialog */
/*************************/
CWallpaperQDlg::CWallpaperQDlg() : CDialog(CWallpaperQDlg::IDD), uiTimer(0)
{
	m_uXPos = 0;
	m_uYPos = 0;
	m_nTransparency = 0;
	m_nBackgroundTransparency = 0;

	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_buildDesktopImage = new CBuildDesktopImage(this->m_hWnd);
	clock = new CClock();
	// Get a pointer to the IActiveDesktop interface.
	// We use this to set the wallpaper when not using .BMP files.
	pIActiveDesktop = NULL;
	HRESULT hr = CoCreateInstance(CLSID_ActiveDesktop, NULL, CLSCTX_INPROC_SERVER, IID_IActiveDesktop, (void**)&pIActiveDesktop);

	if (FAILED(hr))
		pIActiveDesktop = NULL;

	bIconsShown = TRUE;
}

CWallpaperQDlg::~CWallpaperQDlg()
{
	if (hAOT != NULL)
	{
		LPClearHook pClearHook = (LPSetHook)GetProcAddress(hAOT, "ClearHook");
		if (pClearHook != NULL)
		{
			(pClearHook)(this->m_hWnd);
		}
		FreeLibrary(hAOT);
		hAOT = NULL;
	}
	delete m_buildDesktopImage;
	delete clock;
	// Remove the system tray icon.
	Shell_NotifyIcon(NIM_DELETE, &niData);

	// Release the IActiveDesktop interface.
	if (pIActiveDesktop != NULL)
	{
		pIActiveDesktop->Release();
		pIActiveDesktop = NULL;
	}
}

void CWallpaperQDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_BROWSELIST, m_lbBrowseList);
	DDX_Control(pDX, IDC_PREVIEW, m_picPreview);
	DDX_Control(pDX, IDC_POSITION, m_comboPosition);
	DDX_Control(pDX, IDC_QLIST, m_lbQList);
	DDX_Control(pDX, IDC_TYPE, m_stType);
	DDX_Control(pDX, IDC_HEIGHT, m_stHeight);
	DDX_Control(pDX, IDC_WIDTH, m_stWidth);
	DDX_Control(pDX, IDC_RESOLUTION, m_stResolution);
	DDX_Control(pDX, IDC_NUMCOLORS, m_stColors);
	DDX_Control(pDX, IDC_LOOKIN, m_edLookIn);
	DDX_Control(pDX, IDC_QLIST, m_lbQList);
	DDX_Control(pDX, IDC_RANDOMIZE, m_chkRandomize);
	DDX_Control(pDX, IDC_APPLY, m_btnApply);
	DDX_Control(pDX, IDC_ARROWUP, m_btnArrowUp);
	DDX_Control(pDX, IDC_ARROWDOWN, m_btnArrowDown);
	DDX_Control(pDX, IDC_HIDESHOWICONS, m_btnHideshowIcons);
	DDX_Control(pDX, IDC_SETCURRENT, m_btnSetCurrent);
	DDX_Text(pDX, IDC_XPOS, m_uXPos);
	DDX_Text(pDX, IDC_YPOS, m_uYPos);
	DDX_Slider(pDX, IDC_TRANSPARENCY, m_nTransparency);
	DDX_Check(pDX, IDC_BACKGROUNDTRANSPARENCY, m_nBackgroundTransparency);
}

BEGIN_MESSAGE_MAP(CWallpaperQDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_STN_CLICKED(IDC_BROWSE, OnStnClickedBrowse)
	ON_LBN_SELCHANGE(IDC_BROWSELIST, OnLbnSelchangeBrowselist)	
	ON_BN_CLICKED(IDC_ADD, OnBnClickedAdd)
	ON_BN_CLICKED(IDC_QLISTEMPTY, OnBnClickedQlistempty)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_APPLY, OnBnClickedApply)
	ON_EN_CHANGE(IDC_LOOKIN, OnEnChangeLookin)
	ON_CBN_SELCHANGE(IDC_POSITION, OnCbnSelchangePosition)
	ON_COMMAND(ID_CONFIGUREWALLPAPERQUEUE, OnShowConfigWindow)
	ON_COMMAND(ID_SETWALLPAPERTONEXTIMAGE, OnSetToNextImage)
	ON_COMMAND(ID_ABOUTWALLPAPERQUEUE, OnAbout)
	ON_COMMAND(ID_OPTIONS, OnOptions)
	ON_COMMAND(ID_EXITWPQ, OnExit)
	ON_MESSAGE(WM_NI_MSG, OnNotifyIconMsg)
	ON_MESSAGE(WM_SHOWOPTIONS, OnShowOptions)
	ON_MESSAGE(WM_HIDEWINDOW, OnHideMainWindow)
	ON_LBN_SELCHANGE(IDC_QLIST, OnLbnSelchangeQlist)
	ON_BN_CLICKED(IDC_REMOVESELECTED, OnBnClickedRemoveselected)
	ON_BN_CLICKED(IDC_ARROWUP, OnBnClickedArrowup)
	ON_BN_CLICKED(IDC_ARROWDOWN, OnBnClickedArrowdown)
	ON_BN_CLICKED(IDC_RANDOMIZE, OnBnClickedRandomize)
	ON_BN_CLICKED(ID_CANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_HIDESHOWICONS, OnBnClickedHideshowicons)
	ON_BN_CLICKED(IDC_SETCURRENT, OnBnClickedSetCurrent)
	ON_BN_CLICKED(IDC_FONT_BTN, OnFontBtn)
	ON_BN_CLICKED(IDC_COLOR_BTN, OnColorBtn)
	ON_EN_CHANGE(IDC_XPOS, OnEnChangeXpos)
	ON_EN_CHANGE(IDC_YPOS, OnEnChangeYpos)
	ON_BN_CLICKED(IDC_BACKGROUNDTRANSPARENCY, &CWallpaperQDlg::OnBnClickedBackgroundtransparency)
END_MESSAGE_MAP()

void CWallpaperQDlg::OnFontBtn()
{
	// TODO: Add your control notification handler code here
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	CClientDC dc(this);
	lf.lfHeight = -MulDiv(m_nFontSize, dc.GetDeviceCaps(LOGPIXELSY), 72);
	strcpy(lf.lfFaceName, m_szFontName.operator LPCTSTR());
	CFontDialog dlg(&lf);
	if (dlg.DoModal() == IDOK)
	{
		m_szFontName = dlg.GetFaceName();
		m_nFontSize = dlg.GetSize() / 10;
		m_crTextColor = dlg.GetColor();
		DrawText();
	}
}

void CWallpaperQDlg::OnColorBtn()
{
	// TODO: Add your control notification handler code here
	CColorDialog dlg;
	if (dlg.DoModal() == IDOK)
	{
		m_crBackColor = dlg.GetColor();
		DrawText();
	}
}

// CWallpaperQDlg message handlers

BOOL CWallpaperQDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	hAOT = LoadLibrary("AOTHook.dll");

	if (hAOT != NULL)
	{
		LPSetHook pSetHook = (LPSetHook)GetProcAddress(hAOT, "SetHook");
		if (pSetHook != NULL)
		{
			(pSetHook)(this->m_hWnd);
		}
	}

	translate();

	m_buildDesktopImage->SetHWnd(this->m_hWnd);
	clock->Start();

	CDC* pDC = GetDC();  // Get the device context of this window.
	CString cstrScreenRes, cstrColors, cstrCmdLine;
	RECT rect;
	DWORD dwRandom = 0;
	int iScreenWidth, iScreenHeight;
	char strBrowsePath[MAX_PATH + 1];
	BOOL bFirstRun = FALSE;

	// Initialize the strBrowsePath string to zeros.
	ZeroMemory(strBrowsePath, sizeof(char) * (MAX_PATH + 1));

	// Add "About..." menu item to system menu.
	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu = m_Helper.MyLoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// Get the command line.
	cstrCmdLine = AfxGetApp()->m_lpCmdLine;

	// Get the application path.
	GetModuleFileName(AfxGetInstanceHandle(), strBrowsePath, MAX_PATH + 1);
	cstrAppPath.Format("%s", strBrowsePath);
	cstrAppPath = cstrAppPath.Left(cstrAppPath.ReverseFind('\\') + 1);

	//// Get startup options.
	iStartup = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "RunAtStartup", 1); 
	iThisUser = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "ThisOrAllUsers", 1);
	iExitAfter = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "ExitAfter", 0);
	iUseTimer = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "UseTimer", 0);
	iMinutes = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "TimerMinutes", 0);

	// Get the screen dimensions.
	iScreenWidth = GetDeviceCaps(pDC->m_hDC, HORZRES);
	iScreenHeight = GetDeviceCaps(pDC->m_hDC, VERTRES);

	// Display resolution and colors.
	cstrScreenRes.Format("%d x %d", iScreenWidth, iScreenHeight);
	m_stResolution.SetWindowText(cstrScreenRes);

	cstrColors.Format("%d bit", GetDeviceCaps(pDC->m_hDC, BITSPIXEL));
	m_stColors.SetWindowText(cstrColors);

	if (pDC)
		ReleaseDC(pDC);  // Release the device context retrieved earlier.

	// Set the scroll size of the browse list box.
	m_lbBrowseList.GetWindowRect(&rect);
	m_lbBrowseList.SetHorizontalExtent(rect.right - rect.left + 100);

	// Set the scroll size of the queue list box.
	m_lbQList.SetHorizontalExtent(m_lbBrowseList.GetHorizontalExtent());

	// Set bitmaps for the arrow buttons.
	m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWUP)));
	m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWDOWN)));

	m_btnHideshowIcons.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_HIDEICONS)));

	// Initialize or reset dialog controls.
	ResetMainDialog();

	// Move the window to the center of the screen.
	GetWindowRect(&rect);
	MoveWindow(iScreenWidth / 2 - (rect.right - rect.left) / 2, iScreenHeight / 2 - (rect.bottom - rect.top) / 2, (rect.right - rect.left),
			   (rect.bottom - rect.top));

	// Initially minimize the dialog if not the first run.
	bFirstRun = (CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "RunAtStartup", 2) == 2);
	if (bFirstRun)
		PostMessage(WM_SHOWOPTIONS, 0, 0);
	else
	{
		// If we send a command line of "-noexit", then keep the window up.
		if (cstrCmdLine.CompareNoCase("-noexit") == 0)
			iExitAfter = 0;
		else
			PostMessage(WM_HIDEWINDOW);
	}

	// Change the wallpaper.
	if (iUseTimer == 1 && iMinutes > 0)
		uiTimer = (UINT)SetTimer(1, iMinutes * 60000, NULL);
	else
		ChangeWallpaper(m_chkRandomize.GetCheck());

	//Update automatic
	iUpdate = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Update", FALSE);
	//Display clock
	iDisplayClock = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "DisplayClock", FALSE);

	m_uXPos = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartX", 100); 
	m_uYPos = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartY", 100);
	m_szFontName = CRegistry::GetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Font Name", _T("Monotype Corsiva"));
	m_nFontSize = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Text Size", 24);
	m_crTextColor = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Text Color", (INT)RGB(0, 255, 0));
	m_crBackColor = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Back Ground Color",
										   (INT)RGB(255, 255, 255));
	m_nTransparency = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Transparency", 88);
	m_nBackgroundTransparency = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "BackgroundTransparency",
													   0);

	CSliderCtrl* pSlider = (CSliderCtrl*)GetDlgItem(IDC_TRANSPARENCY);
	pSlider->SetRange(0, 255, true);
	pSlider->SetPos(m_nTransparency);

	// Display the tray icon.
	niData.cbSize = NOTIFYICONDATA_V1_SIZE;
	niData.hWnd = this->m_hWnd;
	niData.uID = IDR_MAINFRAME;
	niData.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	niData.uCallbackMessage = WM_NI_MSG;
	niData.hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	strcpy(niData.szTip, "Wallpaper Q");

	Shell_NotifyIcon(NIM_ADD, &niData);
	m_buildDesktopImage->SetIcon();

	if (iUpdate == TRUE)
	{
		CString buf;
		CAutoUpdater updater;
		CAutoUpdater::ErrorType err = updater.CheckForUpdate(_T("http://systrayplay.free.fr/"));
		switch (err)
		{
		case CAutoUpdater::Success:
			buf = m_Helper.MyLoadString(IDS_UPDATEOK);
			AfxMessageBox(buf, MB_ICONINFORMATION | MB_OK);
			//			::PostMessage(pFrame->m_hWnd, WM_DESTROY, 0, 0);
			break;
		case CAutoUpdater::UpdateNotRequired:
			break;
		case CAutoUpdater::UpdateNotComplete:
			buf = m_Helper.MyLoadString(IDS_UPDATEKO);
			AfxMessageBox(buf, MB_ICONINFORMATION | MB_OK);
			break;
		case CAutoUpdater::NoExecutableVersion:
			buf = m_Helper.MyLoadString(IDS_UPDATENOEXECUTABLEVERSION);
			AfxMessageBox(buf, MB_ICONINFORMATION | MB_OK);
			break;
		default:
			buf = m_Helper.MyLoadString(IDS_UPDATECONNECTFAILED);
			AfxMessageBox(buf, MB_ICONINFORMATION | MB_OK);
			break;
		}
	}
	UpdateData(false);
	return TRUE;
}

void CWallpaperQDlg::translate()
{
	m_Helper.translateWindowText(this, IDD_WALLPAPERQ_DIALOG);
	m_Helper.translateDlgItemText(this, IDC_LLOOKON);
	m_Helper.translateDlgItemText(this, IDC_ADD);
	m_Helper.translateDlgItemText(this, IDC_LLOOK);
	m_Helper.translateDlgItemText(this, IDC_COLOR);
	m_Helper.translateDlgItemText(this, IDC_INFOIMG);
	m_Helper.translateDlgItemText(this, IDC_WIDTH);
	m_Helper.translateDlgItemText(this, IDC_HEIGHT);
	m_Helper.translateDlgItemText(this, IDC_IMGQUEUED);
	m_Helper.translateDlgItemText(this, IDC_TYPE);
	m_Helper.translateDlgItemText(this, IDC_POS);
	m_Helper.translateDlgItemText(this, IDC_RANDOMIZE);
	m_Helper.translateDlgItemText(this, IDC_SETCURRENT);
	m_Helper.translateDlgItemText(this, IDC_REMOVESELECTED);
	m_Helper.translateDlgItemText(this, IDC_QLISTEMPTY);
	m_Helper.translateDlgItemText(this, IDC_WALLPAPERTITTLE);
	m_Helper.translateDlgItemText(this, IDC_CALENDAR);
	m_Helper.translateDlgItemText(this, IDC_LLANGUAGE);
	m_Helper.translateDlgItemText(this, IDC_POSX);
	m_Helper.translateDlgItemText(this, IDC_POSY);
	m_Helper.translateDlgItemText(this, IDC_FONT);
	m_Helper.translateDlgItemText(this, IDC_FONT2);
	m_Helper.translateDlgItemText(this, IDC_FONT3);
	m_Helper.translateDlgItemText(this, IDC_BACKGROUNDTRANSPARENCY);
	m_Helper.translateDlgItemText(this, IDOK);
	m_Helper.translateDlgItemText(this, ID_CANCEL);
	m_Helper.translateDlgItemText(this, IDC_APPLY);

	m_comboPosition.AddString(m_Helper.MyLoadString(IDS_TILED));
	m_comboPosition.AddString(m_Helper.MyLoadString(IDS_CENTER));
	m_comboPosition.AddString(m_Helper.MyLoadString(IDS_STRETCH));
}

void CWallpaperQDlg::OnPaint()
{
	CDialog::OnPaint();

	// Make sure the preview gets redrawn.
	DrawPreview();
	DrawText();
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CWallpaperQDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CWallpaperQDlg::ResetMainDialog()
{
	// Get the browse path for the top list box.
	cstrBrowsePath = CRegistry::GetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "LastPath", cstrAppPath);

	m_edLookIn.SetWindowText(cstrBrowsePath);

	// Fill the top list box with all image files in the specified location.
	m_lbBrowseList.ResetContent();
	FillList();

	// Fill the queue list with currently queued images.
	FillQueuedImages();

	// Disable the position combobox since nothing is selected in the queue list. 
	m_comboPosition.EnableWindow(FALSE);

	// Set the randomize checkbox.
	m_chkRandomize.SetCheck((int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Randomize"));

	m_btnApply.EnableWindow(FALSE);
}

void CWallpaperQDlg::FillQueuedImages()
{
	DWORD dwNumImagesInQueue = 0;
	CString cstrImageInfoPath = "";
	CString cstrImageInfo = "";
	int iQueuedWallpaperPosition = 0;

	// Get the total number of already queued images from the registry.
	dwNumImagesInQueue = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "NumImages");

	// Clear the queued image list.
	m_lbQList.ResetContent();

	// Fill the queued images list with already queued images from the registry.
	for (DWORD dwLoop = 0; dwLoop < dwNumImagesInQueue; dwLoop++)
	{
		cstrImageInfoPath.Format("%d", dwLoop);

		cstrImageInfo = CRegistry::GetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", cstrImageInfoPath);

		// We save the image path and also the "position" of the wallpaper in the same string (comma delimited).
		int iIndex = m_lbQList.InsertString((int)dwLoop, cstrImageInfo.Left(cstrImageInfo.Find(",")));
		int iWPPos = atoi(cstrImageInfo.Right(cstrImageInfo.GetLength() - cstrImageInfo.Find(",") - 1));
		m_lbQList.SetItemData(iIndex, iWPPos);
	}

	// Set the currently displayed wallpaper item.
	iCurrentImage = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "CurrentImage");

	// Select the first item.
	if (dwNumImagesInQueue > 0)
	{
		m_lbQList.SetSel(iCurrentImage);
		m_lbQList.SetFocus();
		OnLbnSelchangeQlist();
	}
}

void CWallpaperQDlg::OnStnClickedBrowse()
{
	BROWSEINFO* lpbi = NULL;
	ITEMIDLIST* lpiidl;
	char chrDiplayName[MAX_PATH];
	char chrPath[MAX_PATH];

	// Open a browse for folder dialog.
	lpbi = new BROWSEINFO;

	ZeroMemory(lpbi, sizeof(BROWSEINFO));
	ZeroMemory(chrPath, sizeof(MAX_PATH));

	lpbi->hwndOwner = this->m_hWnd; 
	lpbi->pidlRoot = NULL; 
	lpbi->pszDisplayName = chrDiplayName;
	CString buf = m_Helper.MyLoadString(IDS_BROWSE);
	char tmp[500];
	strcpy(tmp, buf.GetBuffer());
	lpbi->lpszTitle = tmp;
	lpbi->ulFlags = BIF_EDITBOX | BIF_RETURNONLYFSDIRS | BIF_BROWSEINCLUDEFILES; 
	lpbi->lpfn = NULL; 

	lpiidl = SHBrowseForFolder(lpbi);

	SHGetPathFromIDList(lpiidl, chrPath);

	// Set the browse path to the path you just browsed to.
	if (chrPath[0] != '\0')
	{
		strcat(chrPath, "\\");
		SetBrowsePath(chrPath);

		m_edLookIn.SetWindowText(GetBrowsePath());

		if (chrPath[0] != '\0')
		{
			// Reset the top list box.
			m_lbBrowseList.ResetContent();
			OnLbnSelchangeBrowselist();

			FillList();
		}
	}

	delete lpbi;
}

void CWallpaperQDlg::OnEnChangeLookin()
{
	// Change browse list contents based on what they typed in the edit box.
	m_edLookIn.GetWindowText(cstrBrowsePath);
	m_lbBrowseList.ResetContent();
	OnLbnSelchangeBrowselist();
	FillList();
}

void CWallpaperQDlg::FillList()
{
	CFileFind finder;
	int iOnlyBMPs = 1;

	// Clear the list box.
	m_lbBrowseList.ResetContent();

	// Build the search string.
	if (cstrBrowsePath.GetAt(cstrBrowsePath.GetLength() - 1) != '\\')
		cstrBrowsePath += "\\";

	CString cstrSearch[6] =
	{
		cstrBrowsePath + "*.bmp", cstrBrowsePath + "*.jpg", cstrBrowsePath + "*.jpeg", cstrBrowsePath + "*.gif", cstrBrowsePath + "*.dib",
		cstrBrowsePath + "*.png"
	};

	// If Active Desktop is available, we can use other images besides BMPs.
	if (pIActiveDesktop != NULL)
		iOnlyBMPs = 6;

	// Get all image files into the list.
	for (int i = 0; i < iOnlyBMPs; i++)
	{
		BOOL bWorking = finder.FindFile(cstrSearch[i]);
		while (bWorking)
		{
			bWorking = finder.FindNextFile();
			m_lbBrowseList.AddString(finder.GetFilePath());
		}

		finder.Close();
	}

	// Select the first image.
	if (m_lbBrowseList.GetCount() > 0)
	{
		m_lbBrowseList.SetSel(0);
		m_lbBrowseList.SetFocus();
		OnLbnSelchangeBrowselist();
	}
}

void CWallpaperQDlg::OnLbnSelchangeBrowselist()
{
	// Change the previewed image.
	DrawPreview();
}

void CWallpaperQDlg::DrawPreview()
{
	USES_CONVERSION;

	CString cstrWidth, cstrHeight, cstrType;
	UINT uiWidth, uiHeight;
	UINT thumbWidth, thumbHeight;
	int iAdjTop, iAdjLeft;
	int iCount = 0;
	double dRatio;
	CDC* pDC;
	TCHAR szBuf[MAX_PATH];
	RECT rect;
	CWnd* pWndTemp = NULL;
	BOOL bExists = TRUE;

	// Initialize the Graphics class in GDI+.
	pDC = m_picPreview.GetWindowDC();
	Graphics graphics(pDC->m_hDC);

	// Fill the preview ara with a WHITE background.
	m_picPreview.GetClientRect(&rect);
	rect.left += 1;
	rect.top += 1;
	rect.right -= 1;
	rect.bottom -= 1;
	pDC->FillSolidRect(&rect, RGB(255, 255, 255));

	// Determine which listbox (LeftList or QList has focus.
	pWndTemp = GetFocus();
	if (pWndTemp)
	{
		// Since one list is a ListBox and the other is a ListCtrl
		// we get the count in 2 different ways.
		if (GetFocus()->m_hWnd == m_lbQList.m_hWnd)
			iCount = m_lbQList.GetCount();  // QList has focus, so get its count.
		else
			iCount = m_lbBrowseList.GetCount();  // LeftList has focus, so get its count.

		if (iCount > 0)
		{
			// Get the path to the image from the list with focus.
			if (GetFocus()->m_hWnd == m_lbQList.m_hWnd)
				m_lbQList.GetText(m_lbQList.GetCurSel(), szBuf);
			else
				m_lbBrowseList.GetText(m_lbBrowseList.GetCurSel(), szBuf);

			// Make sure there is an image chosen.
			if (szBuf[0] != '/0')
			{
				// Make sure the image exists
				CFile file;
				if (file.Open(szBuf, CFile::modeRead))
					file.Close();
				else
					bExists = FALSE;

				if (bExists)
				{
					// Use the Image class to display a thumbnail of the image.
					Image image(T2CW(szBuf));

					// Determine the appropriate size of the thumbnail preview
					// given the image size ratio and the preview window size ratio.
					uiWidth = image.GetWidth();
					uiHeight = image.GetHeight();
					dRatio = ((double)uiWidth) / ((double)uiHeight);

					// If the width is larger than the height of the image, 
					// set the width of the thumbnail to the width of the preview area
					// and calculate the thumbnail height by using the ratios.
					// If the height is larger than the width of the image, 
					// set the height of the thumbnail to the height of the preview area
					// and calculate the thumbnail width by using the ratios.
					if (uiWidth > uiHeight)
					{
						thumbWidth = rect.right - rect.left;
						thumbHeight = (UINT)(thumbWidth / dRatio);

						if (thumbHeight == 0)
							thumbHeight = 1;
						if (thumbHeight > (UINT)(rect.bottom - rect.top))
							thumbHeight = rect.bottom - rect.top;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1 + ((rect.bottom - rect.top) / 2) - (thumbHeight / 2);
						iAdjLeft = 1;
					}
					else
					{
						thumbHeight = rect.bottom - rect.top;
						thumbWidth = (UINT)(uiWidth * thumbHeight / uiHeight);

						if (thumbWidth == 0)
							thumbWidth = 1;
						if (thumbWidth > (UINT)(rect.right - rect.left))
							thumbWidth = rect.right - rect.left;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1;
						iAdjLeft = 1 + ((rect.right - rect.left) / 2) - (thumbWidth / 2);
					}

					// Get the thumbnail and display it in the preview control.
					Image* pThumbnail = image.GetThumbnailImage(thumbWidth, thumbHeight, NULL, NULL);
					graphics.DrawImage(pThumbnail, iAdjLeft, iAdjTop, pThumbnail->GetWidth(), pThumbnail->GetHeight());

					// Display the image width and height.
					CString buf = m_Helper.MyLoadString(IDC_WIDTH);
					cstrWidth.Format("%s %d", buf, uiWidth);
					m_stWidth.SetWindowText(cstrWidth);
					buf = m_Helper.MyLoadString(IDC_HEIGHT);
					cstrHeight.Format("%s %d", buf, uiHeight);
					m_stHeight.SetWindowText(cstrHeight);

					// Display the image bits per pixel (color depth).
					buf = m_Helper.MyLoadString(IDC_TYPE);
					switch (image.GetPixelFormat())
					{
					case PixelFormat1bppIndexed:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_1BPP));
						break;
					case PixelFormat4bppIndexed:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_4BPP));
						break;
					case PixelFormat8bppIndexed:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_8BPP));
						break;
					case PixelFormat16bppARGB1555:
					case PixelFormat16bppGrayScale:
					case PixelFormat16bppRGB555:
					case PixelFormat16bppRGB565:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_16BPP));
						break;
					case PixelFormat24bppRGB:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_24BPP));
						break;
					case PixelFormat32bppARGB:
					case PixelFormat32bppPARGB:
					case PixelFormat32bppRGB:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_32BPP));
						break;
					case PixelFormat48bppRGB:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_48BPP));
						break;
					case PixelFormat64bppARGB:
					case PixelFormat64bppPARGB:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_64BPP));
						break;
					default:
						cstrType.Format("%s %s", buf, m_Helper.MyLoadString(IDS_UNKNOWN));
						break;
					}

					m_stType.SetWindowText(cstrType);
				}
				else
				{
					COLORREF oldCR = pDC->SetTextColor(RGB(255, 0, 0));
					//TODO
					pDC->DrawText(m_Helper.MyLoadString(IDS_FILENOTEXIST), &rect, DT_CENTER | DT_WORDBREAK);
					pDC->SetTextColor(oldCR);

					// No image to preview.
					CString buf = m_Helper.MyLoadString(IDC_WIDTH);
					m_stWidth.SetWindowText(buf);
					buf = m_Helper.MyLoadString(IDC_HEIGHT);
					m_stHeight.SetWindowText(buf);
					buf = m_Helper.MyLoadString(IDC_TYPE);
					m_stType.SetWindowText(buf);
				}
			}
			else
			{
				// No image to preview.
				CString buf = m_Helper.MyLoadString(IDC_WIDTH);
				m_stWidth.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_HEIGHT);
				m_stHeight.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_TYPE);
				m_stType.SetWindowText(buf);
			}
		}
	}

	if (pDC)
		ReleaseDC(pDC);  // Release the device context retrieved earlier.
}

void CWallpaperQDlg::OnBnClickedAdd()
{
	int iCount;
	CString cstrTemp;

	// Get all selected items from the browse list.
	iCount = m_lbBrowseList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbBrowseList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and add it to the Queue list.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
	{
		m_lbBrowseList.GetText(aryListBoxSel.GetAt(i), cstrTemp);

		// Select each newly added item.
		if (m_lbQList.FindStringExact(0, cstrTemp) > -1)
		{
			// If its already in the list, ask if they want to add it again.
			if (AfxMessageBox(m_Helper.MyLoadString(IDS_ALREADYEXIST), MB_YESNO) == IDYES)
			{
				int index = m_lbQList.InsertString(m_lbQList.GetCount() + 1, cstrTemp);
				m_lbQList.SetItemData(index, GetWPPositionByImageSize(cstrTemp));
			}
		}
		else
		{
			//int index = m_lbQList.InsertString(m_lbQList.GetCount() + 1, cstrTemp);
			int index = m_lbQList.AddString(cstrTemp);
			m_lbQList.SetItemData(index, GetWPPositionByImageSize(cstrTemp));
		}
	}

	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnLbnSelchangeQlist()
{
	// Show the last selected item in the queued image list in the preview window.
	DrawPreview();

	if (m_lbQList.GetCount() > 0)
	{
		// If there are items in the queue list, enable the position combo box. 
		m_comboPosition.SetCurSel((int)m_lbQList.GetItemData(m_lbQList.GetCurSel()));
		m_comboPosition.EnableWindow();

		// Change the up and down arrow images.
		if (m_lbQList.GetSelCount() > 1)
		{
			m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWUP)));
			m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWDOWN)));
		}
		else if (m_lbQList.GetSelCount() == 1)
		{
			m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_ARROWUP)));
			m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_ARROWDOWN)));
		}
	}
}

void CWallpaperQDlg::OnBnClickedArrowup()
{
	int iIndex;
	DWORD dwData;
	CString cstrPath;

	// Move the selected item (single selection only) up one spot in the list.
	if (m_lbQList.GetCount() > 0 && m_lbQList.GetSelCount() == 1)
	{
		iIndex = m_lbQList.GetCurSel();

		if (iIndex > 0)
		{
			m_lbQList.GetText(iIndex, cstrPath);
			dwData = (DWORD)m_lbQList.GetItemData(iIndex);

			m_lbQList.DeleteString(iIndex);

			iIndex--;

			m_lbQList.InsertString(iIndex, cstrPath);
			m_lbQList.SetItemData(iIndex, dwData);

			m_lbQList.SetSel(iIndex);
			m_lbQList.SetFocus();
			OnLbnSelchangeQlist();

			m_btnApply.EnableWindow();
		}
	}
}

void CWallpaperQDlg::OnBnClickedArrowdown()
{
	int iIndex;
	DWORD dwData;
	CString cstrPath;

	// Move the selected item (single selection only) down one spot in the list.
	if (m_lbQList.GetSelCount() > 0 && m_lbQList.GetSelCount() == 1)
	{
		iIndex = m_lbQList.GetCurSel();

		if (iIndex < m_lbQList.GetCount())
		{
			m_lbQList.GetText(iIndex, cstrPath);
			dwData = (DWORD)m_lbQList.GetItemData(iIndex);

			m_lbQList.DeleteString(iIndex);

			iIndex++;

			m_lbQList.InsertString(iIndex, cstrPath);
			m_lbQList.SetItemData(iIndex, dwData);

			m_lbQList.SetSel(iIndex);
			m_lbQList.SetFocus();
			OnLbnSelchangeQlist();

			m_btnApply.EnableWindow();
		}
	}
}


void CWallpaperQDlg::OnBnClickedRemoveselected()
{
	int iCount = 0;

	// Get all selected items from the browse list.
	iCount = m_lbQList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbQList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and delete it from the Queue list.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
		m_lbQList.DeleteString(aryListBoxSel.GetAt(i));

	m_btnApply.EnableWindow();
}


void CWallpaperQDlg::OnBnClickedQlistempty()
{
	// Empty the Queued images list.
	m_lbQList.ResetContent();

	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnCbnSelchangePosition()
{
	int iPosition = 1;
	int iCount = 0;

	iPosition = m_comboPosition.GetCurSel();

	// Get all selected items from the queued image list.
	iCount = m_lbQList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbQList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and change its wallpaper position.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
		m_lbQList.SetItemData(aryListBoxSel.GetAt(i), (DWORD)iPosition);

	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnBnClickedRandomize()
{
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnSetToNextImage()
{
	ApplyChanges();
}

void CWallpaperQDlg::ApplyChanges(BOOL bChangeWallpaper /*= TRUE*/)
{
	// Delete all keys under wallpaperq in the registry.
	CRegistry::RegDelete("Software\\Desktop Calendar_V1.0", "", "WallpaperQ");

	// Set the last browse path in the registry.
	CRegistry::SetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "LastPath", cstrBrowsePath);

	// Set the number of images saved in the registry from the queue list.
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "NumImages", m_lbQList.GetCount());

	// Loop through the queued image list and save each image in the registry.
	for (int iLoop = 0; iLoop < m_lbQList.GetCount(); iLoop++)
	{
		CString cstrPath;
		CString cstrRegValue = "";
		CString cstrRegPath = "";

		m_lbQList.GetText(iLoop, cstrPath);
		cstrRegPath.Format("%s,%d", cstrPath, (int)m_lbQList.GetItemData(iLoop));

		cstrRegValue.Format("%d", iLoop);

		CRegistry::SetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", cstrRegValue, cstrRegPath);
	}

	// Set whether to display a random image when run.
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Randomize", (DWORD)m_chkRandomize.GetCheck());

	// Set them in the registry.
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "RunAtStartup", (DWORD)iStartup); 
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "ThisOrAllUsers", (DWORD)iThisUser);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "ExitAfter", (DWORD)iExitAfter);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "UseTimer", (DWORD)iUseTimer);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "TimerMinutes", (DWORD)iMinutes);

	CRegistry::SetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Font Name", m_szFontName);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Text Size", m_nFontSize);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Text Color", m_crTextColor);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Back Ground Color", m_crBackColor);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Transparency", m_nTransparency);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "BackgroundTransparency", m_nBackgroundTransparency);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartX", m_uXPos);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartY", m_uYPos);


	// Change the wallpaper.
	if (bChangeWallpaper)
		ChangeWallpaper(m_chkRandomize.GetCheck());
}

void CWallpaperQDlg::OnBnClickedApply()
{
	// Apply changes without closing/minimizing.
	UpdateData(true);
	ApplyChanges();

	m_btnApply.EnableWindow(FALSE);
}

void CWallpaperQDlg::ChangeWallpaper(int iMode)
{
	USES_CONVERSION;

	int iImageNum = 0;
	int iPosition = 1;
	CString cstrTileWallpaper;
	CString cstrWallpaperStyle;
	TCHAR szBuf[MAX_PATH];
	m_buildDesktopImage->ReadRegistry();
	m_buildDesktopImage->StartBitmap();

	// Choose then next image in the list, selected image, or choose a random image.
	if (iMode == 1)  // random
	{
		srand((unsigned)time(NULL));

		iImageNum = rand() % m_lbQList.GetCount();

		iCurrentImage = iImageNum;
	}
	else if (iMode == 2)
	{
		iCurrentImage = m_lbQList.GetCurSel();

		iImageNum = iCurrentImage;
	}
	else
	{
		iCurrentImage++;
		if (iCurrentImage + 1 > m_lbQList.GetCount())
			iCurrentImage = 0;

		iImageNum = iCurrentImage;
	}
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "CurrentImage", (DWORD)iCurrentImage);

	for (int i = 0; i< m_buildDesktopImage->m_MonitorArray.GetSize(); i++)
	{
		if (i > 0)
		{
			// Choose then next image in the list, selected image, or choose a random image.
			if (iMode == 1)  // random
			{
				iImageNum = rand() % m_lbQList.GetCount();
			}
			else if (iMode == 2)
			{
				iImageNum++;
				if (iImageNum + 1 > m_lbQList.GetCount())
					iImageNum = 0;
			}
			else
			{
				iImageNum++;
				if (iImageNum + 1 > m_lbQList.GetCount())
					iImageNum = 0;
			}
		}
		// Get the image path from the registry by the image number.
		iPosition = (int)m_lbQList.GetItemData(iImageNum);
		m_lbQList.GetText(iImageNum, szBuf);
		CString tmp;
		tmp.Format("%s", szBuf);
		m_buildDesktopImage->SetImage(tmp, iPosition, i);
	}

	m_buildDesktopImage->StopBitmap();

	CHAR szSysDir[MAX_PATH];
	GetSystemDirectory(szSysDir, MAX_PATH);
	CString szNewFile = szSysDir;
	szNewFile += _T("\\Desktop Calendar.bmp");
	strcpy(szBuf, szNewFile.operator LPCTSTR());

	if (szBuf[0] != '/0')
	{
		// If Active Desktop is not available, use the SystemParametersInfo function.
		if (pIActiveDesktop == NULL)
		{
			if (iPosition == 0)
			{
				// Tiled
				cstrTileWallpaper = "1";
				cstrWallpaperStyle = "1";
			}
			else if (iPosition == 1)
			{
				// Center
				cstrTileWallpaper = "0";
				cstrWallpaperStyle = "1";
			}
			else
			{
				// Stretch
				cstrTileWallpaper = "0";
				cstrWallpaperStyle = "2";
			}
			cstrTileWallpaper = "1";
			cstrWallpaperStyle = "1";

			// Set registry values for the wallpaper position.
			CRegistry::SetRegString("Control Panel\\Desktop", "TileWallpaper", cstrTileWallpaper);
			CRegistry::SetRegString("Control Panel\\Desktop", "WallpaperStyle", cstrWallpaperStyle);

			// Change the wallpaper.
			SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, (void*)szBuf, SPIF_UPDATEINIFILE | SPIF_SENDWININICHANGE);
		}
		else
		{
			WALLPAPEROPT wpOptions;
			COMPONENTSOPT compOptions;

			compOptions.dwSize = sizeof(COMPONENTSOPT);
			compOptions.fActiveDesktop = TRUE;
			compOptions.fEnableComponents = TRUE;

			pIActiveDesktop->SetDesktopItemOptions(&compOptions, 0);

			// Set the wallpaper position.
			wpOptions.dwSize = sizeof(WALLPAPEROPT);
			if (iPosition == 0)
				wpOptions.dwStyle = WPSTYLE_TILE;
			else if (iPosition == 1)
				wpOptions.dwStyle = WPSTYLE_CENTER;
			else
				wpOptions.dwStyle = WPSTYLE_STRETCH;
			wpOptions.dwStyle = WPSTYLE_TILE;

			pIActiveDesktop->SetWallpaperOptions(&wpOptions, 0);

			// Set the wallpaper image.
			pIActiveDesktop->SetWallpaper(T2CW(szBuf), 0);

			// Apply all changes.
			pIActiveDesktop->ApplyChanges(AD_APPLY_ALL);
		}

		if (iExitAfter == 1)
			OnExit();
	}
}

LRESULT CWallpaperQDlg::OnNotifyIconMsg(WPARAM wParam, LONG lParam)
{
	// Show a right-click menu from the system tray icon 
	// if they right-click or double click it.
	switch (lParam)
	{
	case WM_LBUTTONDBLCLK:
		SetForegroundWindow();
		OnShowConfigWindow();
		break;
	case WM_RBUTTONUP:
		POINT point;
		CMenu menu;

		GetCursorPos(&point);

		menu.LoadMenu(IDR_NOTIFYICON_MENU);
		m_Helper.translateMenu(&menu, IDR_NOTIFYICON_MENU);
		CMenu* pMenu = menu.GetSubMenu(0);
		pMenu->SetDefaultItem(ID_CONFIGUREWALLPAPERQUEUE);

		SetForegroundWindow();

		pMenu->TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);

		PostMessage(WM_NULL, 0, 0);
		menu.DestroyMenu();
		break;
	}

	return 1;
}

void CWallpaperQDlg::OnAbout()
{
	// Show the about dialog.
	CAboutDlg dlgAbout;
	dlgAbout.DoModal();
}

void CWallpaperQDlg::OnShowConfigWindow()
{
	RECT trayrect;

	// Set the currently displayed wallpaper item.
	iCurrentImage = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "CurrentImage");

	// Select it in the list
	m_lbQList.SetSel(iCurrentImage);
	m_lbQList.SetFocus();
	OnLbnSelchangeQlist();

	GetTrayWndRect(&trayrect);

	DrawAnimatedRects(IDANI_CAPTION, &trayrect, &rcLastSize);

	MoveWindow(&rcLastSize);
}

LRESULT CWallpaperQDlg::OnHideMainWindow(WPARAM wParam, LONG lParam)
{
	RECT rcTray, rcWindow;

	GetTrayWndRect(&rcTray);
	GetWindowRect(&rcWindow);

	rcLastSize = rcWindow;

	MoveWindow(rcTray.left, rcTray.top, 0, 0);

	DrawAnimatedRects(IDANI_CAPTION, &rcWindow, &rcTray);

	return 1;
}

LRESULT CWallpaperQDlg::OnShowOptions(WPARAM wParam, LONG lParam)
{
	// Display the options dialog.
	OnOptions();

	return 1;
}

void CWallpaperQDlg::OnOptions()
{
	CString cstrStartupApp = "";

	// Get initial options values from the registry.
	iStartup = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "RunAtStartup", 1); 
	iThisUser = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "ThisOrAllUsers", 1);
	iExitAfter = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "ExitAfter", 0);
	iUseTimer = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "UseTimer", 0);
	iMinutes = (int)CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "TimerMinutes", 0);

	// Open the options dialog, sending the options values.
	COptionsDialog dlgOptions(iStartup, iThisUser, iExitAfter, iUseTimer, iMinutes, iUpdate, iDisplayClock);
	if (dlgOptions.DoModal() == IDOK)
	{
		// Get the chosen options.
		iStartup = dlgOptions.GetRunAtStartup(); 
		iThisUser = dlgOptions.GetRunForThisUserOrAll();
		iExitAfter = dlgOptions.GetExitAfterWallpaperChange();
		iUseTimer = dlgOptions.GetUseTimer();
		iMinutes = dlgOptions.GetMinutesForTimer();
		iUpdate = dlgOptions.GetUpdate();
		iDisplayClock = dlgOptions.GetDisplayClock();
		clock->ShowWindowClock(iDisplayClock);
		CRegistry::SetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Language", dlgOptions.Language); 
		translate();

		// Set them in the registry.
		CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "RunAtStartup", (DWORD)iStartup); 
		CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "ThisOrAllUsers", (DWORD)iThisUser);
		CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "ExitAfter", (DWORD)iExitAfter);
		CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "UseTimer", (DWORD)iUseTimer);
		CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "TimerMinutes", (DWORD)iMinutes);

		//TODO
		CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Update", (DWORD)iUpdate);
		CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "DisplayClock", (DWORD)iDisplayClock);

		// Remove wallpaperq from the startup portion of the registry.
		CRegistry::RegDelete("Software\\Microsoft\\Windows\\CurrentVersion\\Run", "WallpaperQ", "", HKEY_LOCAL_MACHINE); 
		CRegistry::RegDelete("Software\\Microsoft\\Windows\\CurrentVersion\\Run", "WallpaperQ", "", HKEY_CURRENT_USER);

		// If the chose to start the app on system startup, add it back to the registry.
		if (iStartup == 1)
		{
			TCHAR path[MAX_PATH];
			::GetModuleFileName(AfxGetApp()->m_hInstance, path, MAX_PATH);
			cstrStartupApp.Format("\"%s\"", path);
			if (iThisUser == 1)
			{
				// For this user only.
				CRegistry::SetRegString("Software\\Microsoft\\Windows\\CurrentVersion\\Run", "WallpaperQ", cstrStartupApp);
			}
			else
			{
				// For all users.
				CRegistry::SetRegString("Software\\Microsoft\\Windows\\CurrentVersion\\Run", "WallpaperQ", cstrStartupApp, HKEY_LOCAL_MACHINE);
			}
		}

		// If they chose to change the wallpaper every X minutes, kill any 
		// previous timers and start a new one based on the minutes entered.
		if (iUseTimer == 1)
		{
			if (uiTimer != 0)
				KillTimer(uiTimer);

			if (iMinutes > 0)
				uiTimer = (UINT)SetTimer(1, iMinutes * 60000, NULL);
		}
		else
		{
			// Or just kill it if they didn't choose to use a timer.
			if (uiTimer != 0)
				KillTimer(uiTimer);
		}

		// Ask them if they want to exit now if they chose to exit after changing wallpaper.
		if (iExitAfter == 1)
		{
			if (AfxMessageBox(m_Helper.MyLoadString(IDS_EXITAFTERCHANGE), MB_YESNO) == IDYES)
				OnExit();
		}
	}
}

void CWallpaperQDlg::OnTimer(UINT nIDEvent)
{
	CDialog::OnTimer(nIDEvent);

	if (nIDEvent == 1)
	{
		// When the timer is fired, change the wallpaper to the next image.
		ChangeWallpaper(m_chkRandomize.GetCheck());
	}
	else if (nIDEvent == 2)
	{
		// show desktop icons after 2 minutes elapsed
		CWnd* pProgMan = FindWindowEx(NULL, NULL, "ProgMan", "Program Manager");
		CWnd* pSHELLDLL_DefView = FindWindowEx(pProgMan->GetSafeHwnd(), NULL, "SHELLDLL_DefView", "");
		CWnd* pFolderView = FindWindowEx(pSHELLDLL_DefView->GetSafeHwnd(), NULL, "SysListView32", "FolderView");
		pFolderView->ShowWindow(SW_SHOW);

		// change bitmap on button
		m_btnHideshowIcons.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_HIDEICONS)));

		KillTimer(2);
	}
}

void CWallpaperQDlg::OnBnClickedOk()
{
	// Apply changes, then minimize.
	ApplyChanges();

	m_btnApply.EnableWindow(FALSE);

	SendMessage(WM_HIDEWINDOW);
}

void CWallpaperQDlg::OnExit()
{
	SendMessage(WM_CLOSE);
}

void CWallpaperQDlg::OnBnClickedCancel()
{
	SendMessage(WM_HIDEWINDOW);

	// They hit cancel, so we reset things.
	// If they clicked OK, it comes to this function, but it should have 
	// saved everything prior toi this.
	ResetMainDialog();
}

int CWallpaperQDlg::GetWPPositionByImageSize(CString cstrImagePath)
{
	USES_CONVERSION;

	CDC* pDC = NULL;
	UINT uiWidth;
	UINT uiHeight;
	int iVertRes;
	int iHorzRes;
	int iTemp;
	double dRatio;
	double dScreenRatio;
	int iCalcPosition = 2;

	// Initialize the Graphics class in GDI+.
	pDC = GetDC();
	Graphics graphics(pDC->m_hDC);

	// Use the Image class to get dimensions of the image.
	Image image(T2CW((LPCTSTR)cstrImagePath));

	uiWidth = image.GetWidth();
	uiHeight = image.GetHeight();
	dRatio = ((double)uiWidth / (double)uiHeight);
	// round it
	iTemp = (int)((dRatio * 100.0) + 0.51);
	dRatio = ((double)iTemp) / 100.0;

	iHorzRes = pDC->GetDeviceCaps(HORZRES);
	iVertRes = pDC->GetDeviceCaps(VERTRES);
	dScreenRatio = ((double)iHorzRes / (double)iVertRes);
	// round it
	iTemp = (int)((dScreenRatio * 100.0) + 0.51);
	dScreenRatio = ((double)iTemp) / 100.0;

	if (dRatio == dScreenRatio || dRatio > 1.0)
		iCalcPosition = 2; // stretch
	else
		iCalcPosition = 1; // center

	ReleaseDC(pDC);

	return iCalcPosition;
}

void CWallpaperQDlg::OnBnClickedHideshowicons()
{
	if (bIconsShown)
	{
		// minimize windows
		MinimizeAllWindows();
		OnBnClickedCancel();

		// hide desktop icons
		CWnd* pProgMan = FindWindowEx(NULL, NULL, "ProgMan", "Program Manager");
		CWnd* pSHELLDLL_DefView = FindWindowEx(pProgMan->GetSafeHwnd(), NULL, "SHELLDLL_DefView", "");
		CWnd* pFolderView = FindWindowEx(pSHELLDLL_DefView->GetSafeHwnd(), NULL, "SysListView32", "FolderView");
		pFolderView->ShowWindow(SW_HIDE);

		// change bitmap on button
		m_btnHideshowIcons.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_SHOWICONS)));

		// set timer to reset the icons after 2 minutes
		SetTimer(2, 120000, NULL);
	}
	else
	{
		// show desktop icons
		CWnd* pProgMan = FindWindowEx(NULL, NULL, "ProgMan", "Program Manager");
		CWnd* pSHELLDLL_DefView = FindWindowEx(pProgMan->GetSafeHwnd(), NULL, "SHELLDLL_DefView", "");
		CWnd* pFolderView = FindWindowEx(pSHELLDLL_DefView->GetSafeHwnd(), NULL, "SysListView32", "FolderView");
		pFolderView->ShowWindow(SW_SHOW);

		// restore all windows
		MinimizeAllWindows(FALSE);

		this->BringWindowToTop();

		// change bitmap on button
		m_btnHideshowIcons.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_HIDEICONS)));
	}

	bIconsShown = !bIconsShown;
}

void CWallpaperQDlg::MinimizeAllWindows(BOOL bMinimize /*= TRUE*/)
{
	// minimize or restore opened windows
	CLSID clsid;
	IUnknown* pUnk = NULL;

	// get class id of Shell.Application to call method
	CLSIDFromProgID(L"Shell.Application", &clsid);

	// create an object of Shell.Application interface
	HRESULT hr = ::CoCreateInstance(clsid, NULL, CLSCTX_SERVER, IID_IUnknown, (void**)&pUnk);

	if (SUCCEEDED(hr))
	{
		// find and invoke the proper method
		IDispatch* pDispApp;
		hr = pUnk->QueryInterface(IID_IDispatch, (void**)&pDispApp);

		if (SUCCEEDED(hr))
		{
			DISPPARAMS dpNoArgs =
			{
				NULL, NULL, 0, 0
			};
			VARIANT vResult;

			IDispatch* pDispMinimizeAll = NULL;
			DISPID dispidMinimizeAll;

			OLECHAR* szFunction = NULL;

			if (bMinimize)
				szFunction = OLESTR("MinimizeAll");
			else
				szFunction = OLESTR("UndoMinimizeALL");

			hr = pDispApp->GetIDsOfNames(IID_NULL, &szFunction, 1, LOCALE_USER_DEFAULT, &dispidMinimizeAll);
			if (SUCCEEDED(hr))
			{
				hr = pDispApp->Invoke(dispidMinimizeAll, IID_NULL, LOCALE_USER_DEFAULT, DISPATCH_METHOD, &dpNoArgs, &vResult, NULL, NULL);
			}

			pDispApp->Release();
		}

		pUnk->Release();
	}
}

void CWallpaperQDlg::OnBnClickedSetCurrent()
{
	// tell the ChangeWallpaper function to set the wallpaper to the current selection inthe Q list
	UpdateData(true);
	ApplyChanges(FALSE);
	ChangeWallpaper(2);

	m_lbQList.SetFocus();
	Invalidate(FALSE);
}

void CWallpaperQDlg::DrawText()
{
	CWnd* pWnd;
	pWnd = GetDlgItem(IDC_SAMPLE_TEXT);
	if (pWnd)
	{
		CDC* pDC = pWnd->GetDC();
		CFont Font;
		Font.CreateFont(m_nFontSize, 0, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET, 0, 0, 0, FF_DONTCARE, m_szFontName); 
		pDC->SelectObject(&Font);
		TEXTMETRIC tmText;
		pDC->GetTextMetrics(&tmText);
		pDC->SetBkMode(TRANSPARENT);
		pDC->SetTextColor(m_crTextColor);

		INT m_nCharWidth = (tmText.tmMaxCharWidth + 2 * tmText.tmAveCharWidth);	
		INT m_nCharHeight = tmText.tmHeight + 4;

		CRect TextRect;
		CRect ClientRect;
		pWnd->GetClientRect(&ClientRect);
		int nStartX = (ClientRect.Width() - m_nCharWidth * 6) / 2;
		if (nStartX < 0)
			nStartX = 2;
		int nStartY = (ClientRect.Height() - m_nCharHeight) / 2;
		if (nStartY < 0)
			nStartY = 2;

		TextRect.left = nStartX; 
		TextRect.top = nStartY ;
		TextRect.right = TextRect.left + m_nCharWidth * 6;
		TextRect.bottom = TextRect.top + m_nCharHeight;
		CBrush Brush;
		Brush.CreateSolidBrush(m_crBackColor);
		pDC->FillRect(&ClientRect, &Brush);
		//TODO
		pDC->DrawText(_T(m_Helper.MyLoadString(IDS_EXEMPLE)), &TextRect, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
	}
}
void CWallpaperQDlg::OnEnChangeXpos()
{
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnEnChangeYpos()
{
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnBnClickedBackgroundtransparency()
{
	m_btnApply.EnableWindow();
}
